#
# hebrew.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Hebrew(Language):

    name = _("Hebrew")
    used_for = "he"
    feature_description = _("Google translation.")



#
# german.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class German(Language):

    name = _("German")
    used_for = "de"
    feature_description = _("Google translation and text-to-speech.")



#!/usr/bin/python3

# JUnitXML to todo.txt Parser
# Ace Z. Alba (acezalba+github@slmail.me)
#

import os
import xml.etree.ElementTree as ET

def parse_junitxml(xml_file):
    tree = ET.parse(xml_file)
    root = tree.getroot()

    test_results = []

    junitxml_errors = [
        "AssertionError",
        "NullPointerException",
        "TimeoutException",
        "IllegalArgumentException",
        "Exception",
        "IndexError",
        "KeyError",
        "TypeError",
        "ValueError"
    ]

    total_tests = len(root.findall('.//testcase'))
    successful_tests = total_tests  # Initialize successful_tests to the total number of tests
    failures_count = 0
    errors_count = 0

    for testcase in root.findall('.//testcase'):
        test_name = testcase.get('name')
        classname = testcase.get('classname', '')
        time = testcase.get('time', '')

        failure = testcase.find('failure')
        error = testcase.find('error')

        if failure is not None:
            failures_count += 1
            successful_tests -= 1  # Decrement successful_tests on failure
            failure_message = failure.get('message', '')

            for error_type in junitxml_errors:
                if error_type in failure_message:
                    test_results.append(
                        f"{classname}::{test_name} time:{time} @failure @{error_type} message:\"{failure_message}\""
                    )
                    break
            else:
                test_results.append(f"{classname}::{test_name} time:{time} @failure message:\"{failure_message}\"")
        elif error is not None:
            errors_count += 1
            successful_tests -= 1  # Decrement successful_tests on error
            error_message = error.get('message', '')

            for error_type in junitxml_errors:
                if error_type in error_message:
                    test_results.append(
                        f"{classname}::{test_name} time:{time} @error @{error_type} message:\"{error_message}\""
                    )
                    break
            else:
                test_results.append(f"{classname}::{test_name} time:{time} @error message:\"{error_message}\"")
        else:
            test_results.append(f"x {classname}::{test_name} time:{time}")

    print("Summary:")
    print(f"Total Tests: {total_tests}")
    print(f"Successful Tests: {successful_tests}")
    print(f"Failures: {failures_count}")
    print(f"Errors: {errors_count}")

    return test_results

def save_to_todo_file(todo_file, test_results):
    with open(todo_file, 'w') as file:
        file.write('\n'.join(test_results))

if __name__ == "__main__":
    script_dir = os.path.dirname(os.path.abspath(__file__))
    junitxml_file = os.path.join(script_dir, os.pardir, "test-results.xml")
    todo_file = os.path.join(script_dir, os.pardir, "test-results.todo.txt")

    test_results = parse_junitxml(junitxml_file)
    save_to_todo_file(todo_file, test_results)
    print(f"Conversion complete. Results saved to test-results.todo.txt")



#
# tajik.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Tajik(Language):

    name = _("Tajik")
    used_for = "tg"
    feature_description = _("Google translation.")



#
# lao.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Lao(Language):

    name = _("Lao")
    used_for = "lo"
    feature_description = _("Google translation.")



#
# pyqt_render_chain.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.filters.latex import Latex
from mnemosyne.libmnemosyne.render_chain import RenderChain
from mnemosyne.libmnemosyne.renderers.html_css import HtmlCss
from mnemosyne.libmnemosyne.filters.furigana import Furigana
from mnemosyne.libmnemosyne.filters.RTL_handler import RTLHandler
#from mnemosyne.libmnemosyne.filters.html5_audio import Html5Audio
#from mnemosyne.libmnemosyne.filters.html5_video import Html5Video
from mnemosyne.libmnemosyne.filters.expand_paths import ExpandPaths
from mnemosyne.libmnemosyne.filters.escape_to_html import EscapeToHtml
from mnemosyne.libmnemosyne.filters.non_latin_font_size_increase import \
     NonLatinFontSizeIncrease
from mnemosyne.pyqt_ui.qt_audio import QtAudio
from mnemosyne.pyqt_ui.qt_video import QtVideo

class PyQtRenderChain(RenderChain):

    id = "default"
    
    filters = [Latex, EscapeToHtml, ExpandPaths, QtAudio,
        QtVideo, RTLHandler, Furigana, NonLatinFontSizeIncrease]
    
    renderers = [HtmlCss]



#
# azure_tts.py <Peter.Bienstman@UGent.be>
#

# Set up your account according to
# https://learn.microsoft.com/en-us/azure/cognitive-services/speech-service/get-started-text-to-speech?tabs=windows%2Cterminal&pivots=programming-language-python

import os

from mnemosyne.libmnemosyne.plugin import Plugin
from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.utils import expand_path
from mnemosyne.libmnemosyne.pronouncer import Pronouncer



class AzurePronouncer(Pronouncer):

    used_for = "ar", "zh", "ja"

    popup_menu_text = "Insert Azure text-to-speech..."

    def download_tmp_audio_file(self, card_type, foreign_text):

        """Returns a temporary filename with the audio."""
        
        from pydub import AudioSegment
        
        import azure.cognitiveservices.speech as speechsdk
        speech_config = speechsdk.SpeechConfig(subscription=os.environ.get('COGNITIVE_SERVICE_KEY'), region=os.environ.get('COGNITIVE_SERVICE_REGION'))
        speech_config.set_speech_synthesis_output_format(speechsdk.SpeechSynthesisOutputFormat.Riff24Khz16BitMonoPcm)


        language_id = self.config().card_type_property(\
            "sublanguage_id", card_type)
        if not language_id:
            language_id = self.config().card_type_property(\
                "language_id", card_type)

        # https://azure.microsoft.com/en-gb/products/cognitive-services/text-to-speech/#features

        if 'zh' in language_id.lower():
            speech_config.speech_synthesis_voice_name = "zh-CN-YunzeNeural"
        if 'ar' in language_id.lower():
            speech_config.speech_synthesis_voice_name = "ar-JO-TaimNeural"
        if 'ja' in language_id.lower():
            speech_config.speech_synthesis_voice_name = "ja-JP-KeitaNeural"

        speech_config.set_speech_synthesis_output_format(\
            speechsdk.SpeechSynthesisOutputFormat.Riff24Khz16BitMonoPcm)

        speech_synthesizer = speechsdk.SpeechSynthesizer(\
            speech_config=speech_config, audio_config=None)

        result = speech_synthesizer.speak_text_async(foreign_text).get()

        if result.reason == speechsdk.ResultReason.SynthesizingAudioCompleted:
            print("Speech synthesized for text [{}]".format(foreign_text))
        elif result.reason == speechsdk.ResultReason.Canceled:
            cancellation_details = result.cancellation_details
            print("Speech synthesis canceled: {}".format(cancellation_details.reason))
            if cancellation_details.reason == speechsdk.CancellationReason.Error:
                if cancellation_details.error_details:
                    print("Error details: {}".format(cancellation_details.error_details))
                    print("Did you set the speech resource key and region values?")
            return
        
        filename_wav = expand_path("__AZURE_TTS__TMP__.wav", 
            self.database().media_dir())
        filename_mp3 = expand_path("__AZURE_TTS__TMP__.mp3", 
            self.database().media_dir())
        stream = speechsdk.AudioDataStream(result)
        stream.save_to_wav_file(filename_wav)
        audio = AudioSegment.from_wav(filename_wav)
        audio.export(filename_mp3, format="mp3")  
        return filename_mp3


class AzureTTSPlugin(Plugin):

    name = "Azure TTS"
    description = "Add Azure text-to-speech."
    components = [AzurePronouncer]
    gui_for_component = {"AzurePronouncer" :
        [("mnemosyne.pyqt_ui.pronouncer_dlg", "PronouncerDlg")]}
    supported_API_level = 3


# Register plugin.

from mnemosyne.libmnemosyne.plugin import register_user_plugin
register_user_plugin(AzureTTSPlugin)



#
# tagalog.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Tagalog(Language):

    name = _("Tagalog (Filipino)")
    used_for = "tl"
    feature_description = _("Google translation.")



#
# audio_player_container.py
#

# Is needed for JavaScript generation.
# Contains the ID's of HTML audioplayers.
# For example the ID "b" of the player
# <audio id="b" controls><source src="one.mp3"><source src="two.mp3"></audio>
# is stored in the member variable players.
class AudioPlayerContainer:
    def __init__(self):
        self.players = [] 



#
# multiple_audiofile_support.py
#

from mnemosyne.web_server.audio_player_container import AudioPlayerContainer

# Global object for code optimization.
# Is used for JavaScript generation.
# Contains the ID's of HTML audioplayers.
ap_container = AudioPlayerContainer() 

# Contains the ID of a HTML audioplayer.
# Is used for JavaScript generation.
class AudioPlayer:
     def __init__(self, id):
        self.id = id

# Inserts html audio tags.
# For example <source src='soundfile.mp3'> becomes  
# <audio id='player_b' controls><source src='soundfile.mp3'></audio>.
class InsertAudioplayerTags:
    def __init__(self, audio_files_counter):
        self.__audio_files_counter = audio_files_counter
        self.players = []    
    def insert_audioplayer_tags(self, text, fact_key):
        str1 = '<audio id="player_{id}" controls>\n' \
                    .format(id = fact_key)
        str2 = '</audio>' 
        index = text.find('<source src=')
        text_audioplayer_inserted = text[:index] + str1 + text[index:]
        text_audioplayer_inserted += str2
        if self.__audio_files_counter > 1:
            # Player has to play >1 file, JavaScript will be inserted.
            player = AudioPlayer(fact_key)
            ap_container.players.append(player)
        return text_audioplayer_inserted


# Inserts JavaScript into the HTML page when an audio player needs to play more 
# than one sound file. For example the audio player with the id "b" has to play
# two sound files. 
# <audio id="b" controls><source src="one.mp3"><source src="two.mp3"></audio>
# So, JavaScript will be added to the page,
# to enable the ability playing both files.
class InsertJavascript:
    def __del__(self):
        ap_container.players.clear() # Reset content for the next page.
    # Inserts JavaScript into the html page :-O            
    def insert_javascript(self, html_page):
        if 1 < len(ap_container.players):
            # Return unchanged page, no need to insert JavaScript
            return html_page 
        # Contains two lines of JavaScript for each audio player.
        # For example
        # var audio_player_b = null;
        # let index_b = { val : 0 }; 
        audio_player_with_index = ""
        # Contains a JavaScript funtion call to initialize 
        # and to start the audio player.
        # For example
        # init_player(audio_player_b, 'player_b', index_b); 
        call_init_player = ""
        for player in ap_container.players: 
            audio_player_with_index += \
                'var audio_player_{id} = null;\n'.format(id = player.id)
            audio_player_with_index += "let index_{id} = {val};\n". \
                            format(id = player.id, val =  "{val : 0}" )
            call_init_player += \
                "init_player(audio_player_{id}, 'player_{id}' , index_{id});\n".format(id = player.id) 
        javascript = """
            <script>
                %s
                function init_player(audio_player, fact_id, index)
                {
                    var audio_player = document.getElementById(fact_id, index);
                    if (null === audio_player) return;
                        
                    if(audio_player.children.length > 1)  
                    {
                        audio_player.addEventListener('ended', function(event)
                        {
                            play.call(this, event, audio_player, index);
                        }, false);
                    }
                    audio_player.src = audio_player.children[index.val].src
                }
                
                var play = function play_playlist(event, audio_player, index)
                {
                    index.val += 1; 
                    audio_player.autoplay = true;
                    if(index.val == audio_player.children.length)
                    {
                        audio_player.autoplay = false;
                        index.val = 0;
                    }
                    audio_player.src = audio_player.children[index.val].src
                }
                %s   
          </script> 		 
          """ % (audio_player_with_index, call_init_player)
        index = html_page.find(b'</body>')
        if -1 == index:
            return html_page
        html_page_javascript_inserted = (html_page[:index] 
                                            + javascript.encode('utf-8')
                                            + html_page[index:])
        return html_page_javascript_inserted



#
# simple_html5_audio.py <Peter.Bienstman@gmail.com>
#

import re
import urllib.request, urllib.parse, urllib.error

from mnemosyne.libmnemosyne.filter import Filter

player_id = 1

re_audio = re.compile(r"""<audio src=\"(.+?)\"(.*?)>""",
    re.DOTALL | re.IGNORECASE)


class SimpleHtml5Audio(Filter):
    def run(self, text, card, fact_key, **render_args):
        if not re_audio.search(text):
            return text
        for match in re_audio.finditer(text):
            filename = urllib.parse.quote(match.group(1).encode("utf-8"), safe="/:")
            filename = urllib.parse.quote(match.group(1), safe="/:")
            # Prevent wsgi from decoding this as as non-unicode behind
            # our back ( https://bugs.python.org/issue16679).
            filename = filename.replace("%", "___-___")
            text = text.replace(match.group(0), "")
            text += "<source src=\"" + filename + "\">" + '\n'
        global player_id
        str1 = '\n<audio id="player_{id}" autoplay controls>\n<source src='.format(id = player_id)
        text = text.replace('<source src=', str1, 1)
        text += '</audio>\n'
        player_id += 1
        return text



#
# furigana.py <Peter.Bienstman@gmail.com>
#

import re

from mnemosyne.libmnemosyne.filter import Filter

class Furigana(Filter):

    ruby_re = r"([\u4E00-\u9FFF]+)\[([\u3040-\u309F]+)\]"

    def run(self, text, card, fact_key, **render_args):
        return re.sub(self.ruby_re, r"<ruby>\1<rt>\2</rt></ruby>", text)



#
# norwegian.py <Peter.Bienstman@gmail.com>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Norwegian(Language):

    name = _("Norwegian")
    used_for = "no"
    feature_description = _("Google translation and text-to-speech.")
